@@ -175,7 +175,7 @@ getME.glmmTMB <- function(object,
          "Z"     = oo.env$data$Z,
          "Zzi"   = oo.env$data$Zzi,
          "Xd"    = oo.env$data$Xd,
-         "theta" = oo.env$parList()$theta ,
+         "theta" = oo.env$parList(object$fit$par, object$fit$parfull)$theta ,
 
          "..foo.." = # placeholder!
            stop(gettextf("'%s' is not implemented yet",
@@ -398,6 +398,24 @@ printDispersion <- function(ff,s) {
     NULL
 }
 
+.tweedie_power <- function(object) {
+    unname(plogis(object$fit$par["thetaf"]) + 1)
+}
+
+## Print family specific parameters
+## @param ff name of family (character)
+## @param object glmmTMB output
+#' @importFrom stats plogis
+printFamily <- function(ff, object) {
+    if (ff == "tweedie") {
+        power <- .tweedie_power(object)
+        cat(sprintf("\nTweedie power parameter: %s",
+                    formatC(power, digits=3)), "\n")
+
+    }
+    NULL
+}
+
 ##' @importFrom lme4 .prt.aictab
 ##' @method print glmmTMB
 ##' @export
@@ -431,8 +449,10 @@ print.glmmTMB <-
   cat(do.call(paste,c(gvec,list(sep=" / "))),fill=TRUE)
 
   if(trivialDisp(x)) {# if trivial print here, else below(~x) or none(~0)
-    printDispersion(x$modelInfo$familyStr,sigma(x))  
-  } 
+    printDispersion(x$modelInfo$family$family,sigma(x))  
+  }
+  ## Family specific parameters
+  printFamily(x$modelInfo$family$family, x)
   ## Fixed effects:
   if(length(cf <- fixef(x)) > 0) {
     cat("\nFixed Effects:\n")
@@ -457,6 +477,9 @@ model.frame.glmmTMB <- function(formula, ...) {
 ##' @export
 residuals.glmmTMB <- function(object, type=c("response", "pearson"), ...) {
     type <- match.arg(type)
+    if(type=="pearson" &((object$call$ziformula != ~0)|(object$call$dispformula != ~1))) {
+        stop("pearson residuals are not implemented for models with zero-inflation or variable dispersion")
+    }
     r <- model.response(object$frame)-fitted(object)
     switch(type,
            response=r,
@@ -473,6 +496,46 @@ residuals.glmmTMB <- function(object, type=c("response", "pearson"), ...) {
            })
 }
 
+## Helper to get CI of simple *univariate monotone* parameter
+## function, i.e. a function of 'fit$par' and/or 'fit$parfull'.
+## Examples: 'sigma.glmmTMB' and some parts of 'VarCorr.glmmTMB'.
+
+##' @importFrom stats qchisq
+.CI_univariate_monotone <- function(object, f, reduce=NULL,
+                                    level=0.95,
+                                    name.prepend=NULL,
+                                    estimate = TRUE) {
+    x <- object
+    par <- x$fit$par
+    i <- seq_along(x$fit$parfull) ## Pointers into long par vector
+    r <- x$obj$env$random
+    if(!is.null(r)) i <- i[-r]    ## Pointers into short par subset
+    sdr <- x$sdr
+    sdpar <- summary(sdr, "fixed")[,2]
+    q <- sqrt(qchisq(level, df=1))
+    ans <- list()
+    x$fit$parfull[i] <- x$fit$par <- par - q * sdpar
+    ans$lower <- f(x)
+    x$fit$parfull[i] <- x$fit$par <- par + q * sdpar
+    ans$upper <- f(x)
+    if (estimate) {
+        ans$Estimate <- f(object)
+    }
+    if(is.null(reduce)) reduce <- function(x) x
+    ans <- lapply(ans, reduce)
+    nm <- names(ans)
+    tmp <- cbind(ans$lower, ans$upper)
+    if (is.null(tmp) || nrow(tmp) == 0L) return (NULL)
+    sort2 <- function(x) if(any(is.nan(x))) x * NaN else sort(x)
+    ans <- cbind( t( apply(tmp, 1, sort2) ) , ans$Estimate )
+    colnames(ans) <- nm
+    if (!is.null(name.prepend))
+        name.prepend <- rep(name.prepend, length.out = nrow(ans))
+    rownames(ans) <- paste(name.prepend,
+                           rownames(ans), sep="")
+    ans
+}
+
 ## copied from 'stats'
 
 format.perc <- function (probs, digits) {
@@ -480,12 +543,37 @@ format.perc <- function (probs, digits) {
     "%")
 }
 
+##' Calculate confidence intervals
+##'
+##' @details
+##' Currently, all confidence intervals are calculated using the
+##' 'wald' method. These intervals are based on the standard errors
+##' calculated for parameters on the scale
+##' of their internal parameterization depending on the family. Derived
+##' quantities such as standard deviation parameters and dispersion
+##' parameters are backtransformed. It follows that confidence
+##' intervals for these derived quantities are asymmetric.
+##'
 ##' @importFrom stats qnorm confint
 ##' @export
+##' @param object \code{glmmTMB} fitted object.
+##' @param parm Specification of a parameter subset \emph{after}
+##'     \code{component} subset has been applied.
+##' @param level Confidence level.
+##' @param method Currently only option is 'wald'.
+##' @param component Which of the three components 'cond', 'zi' or
+##'     'other' to select. Default is to select 'all'.
+##' @param estimate Logical; Add a 3rd column with estimate ?
+##' @param ... Not used
+##' @examples
+##' data(sleepstudy, package="lme4")
+##' model <- glmmTMB(Reaction ~ Days + (1|Subject), sleepstudy)
+##' confint(model)
 confint.glmmTMB <- function (object, parm, level = 0.95,
-                             method=c("Wald","wald",  ## ugh -- allow synonyms?
+                             method=c("wald",
                                       "profile"),
-                             component= "cond", ...) 
+                             component = c("all", "cond", "zi", "other"),
+                             estimate = TRUE,...)
 {
     dots <- list(...)
     if (length(dots)>0) {
@@ -497,23 +585,68 @@ confint.glmmTMB <- function (object, parm, level = 0.95,
         }
     }
     method <- match.arg(method)
-    cf <- unlist(fixef(object)[component])
-    pnames <- names(cf)
-    if (missing(parm)) 
-        parm <- pnames
-    else if (is.numeric(parm)) 
-        parm <- pnames[parm]
+    components <- match.arg(component, several.ok = TRUE)
+    components.has <- function(x)
+        any(match(c(x, "all"), components, nomatch=0L)) > 0L
     a <- (1 - level)/2
     a <- c(a, 1 - a)
     pct <- format.perc(a, 3)
     fac <- qnorm(a)
-    ci <- array(NA, dim = c(length(parm), 2L), dimnames = list(parm, 
-        pct))
+    estimate <- as.logical(estimate)
+    ci <- matrix(NA, 0, 2 + estimate,
+                 dimnames=list(NULL,
+                               c(pct, "Estimate")
+                               [c(TRUE, TRUE, estimate)] ))
     if (tolower(method)=="wald") {
-        vv <- vcov(object)[component]
-        ss <- unlist(lapply(vv,diag))
-        ses <- sqrt(ss)[parm]
-        ci[] <- cf[parm] + ses %o% fac
+        for (component in c("cond", "zi") ) {
+            if (components.has(component)) {
+                cf <- unlist(fixef(object)[component])
+                vv <- vcov(object)[component]
+                ss <- unlist(lapply(vv,diag))
+                ses <- sqrt(ss)
+                ci.tmp <- cf + ses %o% fac
+                if (estimate) ci.tmp <- cbind(ci.tmp, cf)
+                ci <- rbind(ci, ci.tmp)
+                ## VarCorr -> stddev
+                reduce <- function(VC) sapply(VC[[component]],
+                                              function(x)attr(x, "stddev"))
+                ci.sd <- .CI_univariate_monotone(object,
+                                                 VarCorr,
+                                                 reduce = reduce,
+                                                 level = level,
+                                                 name.prepend=paste(component,
+                                                                    "Std.Dev.",
+                                                                    sep="."),
+                                                 estimate = estimate)
+                ci <- rbind(ci, ci.sd)
+            }
+        }
+        if (components.has("other")) {
+            ## sigma
+            ff <- object$modelInfo$family$family
+            if (usesDispersion(ff)) {
+                ci.sigma <- .CI_univariate_monotone(object,
+                                                    sigma,
+                                                    reduce = NULL,
+                                                    level=level,
+                                                    name.prepend="sigma",
+                                                    estimate = estimate)
+                ci <- rbind(ci, ci.sigma)
+            }
+            ## Tweedie power
+            if (ff == "tweedie") {
+                ci.power <- .CI_univariate_monotone(object,
+                                                    .tweedie_power,
+                                                    reduce = NULL,
+                                                    level=level,
+                                                    name.prepend="Tweedie.power",
+                                                    estimate = estimate)
+                ci <- rbind(ci, ci.power)
+            }
+        }
+        ## Take subset
+        if (!missing(parm))
+            ci <- ci[parm, , drop=FALSE]
     } else {
         stop("profile CI not yet implemented")
         ## FIXME: compute profile(object)
@@ -631,7 +764,7 @@ fitted.glmmTMB <- function(object, ...) {
     predict(object)
 }
 
-.noSimFamilies <- c("beta", "betabinomial", "genpois")
+.noSimFamilies <- c("beta", "genpois")
 
 noSim <- function(x) {
     !is.na(match(x, .noSimFamilies))
@@ -647,6 +780,7 @@ noSim <- function(x) {
 ##' Currently, it is not possible to condition on estimated random effects.  
 ##' @return returns a list of vectors. The list has length \code{nsim}. 
 ##' Each simulated vector of observations is the same size as the vector of response variables in the original data set.
+##' In the binomial family case each simulation is a two-column matrix with success/failure.
 ##' @importFrom stats simulate
 ##' @export
 simulate.glmmTMB<-function(object, nsim=1, seed=NULL, ...){
@@ -655,6 +789,15 @@ simulate.glmmTMB<-function(object, nsim=1, seed=NULL, ...){
     	stop("Simulation code has not been implemented for this family")
     }
     if(!is.null(seed)) set.seed(seed)
-    ret <- replicate(nsim, object$obj$simulate()$yobs, simplify=FALSE)
+    family <- object$modelInfo$family$family
+    ret <- replicate(nsim,
+                     object$obj$simulate(par = object$fit$parfull)$yobs,
+                     simplify=FALSE)
+    if ( binomialType(family) ) {
+        size <- object$obj$env$data$weights
+        ret <- lapply(ret, function(x) cbind(x, size - x, deparse.level=0) )
+    }
+    names(ret) <- paste("sim", seq_len(nsim), sep="_")
+    ret <- as.data.frame(ret)
     ret
 }
